# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 15.11.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 11, task 1.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation
# pylint: disable=redefined-outer-name
# pylint: disable=import-error

import os

import numpy as np
import matplotlib.pyplot as plt
import scipy.signal as signal

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None,
               fixed_range=True):
  """Plot image"""
  fig = plt.figure(fig_num)
  if fixed_range:
    vmin = np.iinfo(image.dtype).min
    vmax = np.iinfo(image.dtype).max
  else:
    vmin = int(np.min(image))
    vmax = int(np.max(image))
  plt.imshow(image, cmap=colormap, interpolation='none', vmin=vmin, vmax=vmax)
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  fig_num += 1
  return fig_num

def plot_lattice(image, fig_num, write_file=None):
  """Plot image with lattice in pixel borders"""
  fig = plt.figure(fig_num)
  ax = fig.add_subplot(111)
  ax.imshow(image, cmap='gray', interpolation='none')
  ax.set_xticks(np.arange(0, image.shape[1]))
  ax.set_xticks(np.arange(0.5, image.shape[1] + 0.5), minor=True)
  ax.get_xaxis().set_ticks([]) # Hide tick labels
  ax.set_yticks(np.arange(0, image.shape[0]))
  ax.set_yticks(np.arange(0.5, image.shape[0] + 0.5), minor=True)
  ax.get_yaxis().set_ticks([]) # Hide tick labels
  ax.grid(b=True, which='minor', linestyle='-', color='lightgray')
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  plt.tight_layout()

  fig_num += 1
  return fig_num

def main():
  """Segmentation of cells"""

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 11')
  print('Task 1')
  print('-'*80)

  # Configurations
  result_dir = 'results/images'

  # Dummy image
  img = np.array([[0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0],
                  [1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
                  [0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0],
                  [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0],
                  [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0],
                  [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]])

  fig_num = plot_lattice(img, 0,
                         write_file=os.path.join(result_dir, 't01_image.png'))

  # One set of masks could be:
  #mask_horizontal = np.array([[-1,  2, -1],
  #                            [-1,  2, -1],
  #                            [-1,  2, -1]])
  #mask_vertical = np.array([[-1, -1, -1],
  #                          [ 2,  2,  2],
  #                          [-1, -1, -1]])
  #mask_diagonal = np.array([[ 2, -1, -1],
  #                          [-1,  2, -1],
  #                          [-1, -1,  2]])
  #mask_antidiagonal = np.array([[-1, -1,  2],
  #                              [-1,  2, -1],
  #                              [ 2, -1, -1]])

  # But if the image is assumed to be noiseless, it is no reason to use masks
  # which are thicker than 1 pixel, so a better set of masks is:
  nl_mask_horizontal = np.array([[0, 0, 0],
                                 [1, -2, 1],
                                 [0, 0, 0]])
  nl_mask_vertical = np.array([[0, 1, 0],
                               [0, -2, 0],
                               [0, 1, 0]])
  nl_mask_diagonal = np.array([[1, 0, 0],
                               [0, -2, 0],
                               [0, 0, 1]])
  nl_mask_antidiagonal = np.array([[0, 0, 1],
                                   [0, -2, 0],
                                   [1, 0, 0]])

  # Detect horizontal gaps
  hor_gaps = signal.convolve2d(img, nl_mask_horizontal, mode='same')
  fig_num = plot_lattice(hor_gaps, fig_num,
                         write_file=os.path.join(result_dir, 't01_hor_gaps.png'))
  fig_num = plot_lattice(hor_gaps > 1, fig_num,
                         write_file=os.path.join(result_dir, 't01_hor_gaps_thresh.png'))
  # Detect vertical gaps
  vert_gaps = signal.convolve2d(img, nl_mask_vertical, mode='same')
  fig_num = plot_lattice(vert_gaps, fig_num,
                         write_file=os.path.join(result_dir, 't01_vert_gaps.png'))
  fig_num = plot_lattice(vert_gaps > 1, fig_num,
                         write_file=os.path.join(result_dir, 't01_vert_gaps_thresh.png'))
  # Detect diagonal gaps
  diag_gaps = signal.convolve2d(img, nl_mask_diagonal, mode='same')
  fig_num = plot_lattice(diag_gaps, fig_num,
                         write_file=os.path.join(result_dir, 't01_diag_gaps.png'))
  fig_num = plot_lattice(diag_gaps > 1, fig_num,
                         write_file=os.path.join(result_dir, 't01_diag_gaps_thresh.png'))
  # Detect antidiagonal gaps
  adiag_gaps = signal.convolve2d(img, nl_mask_antidiagonal, mode='same')
  fig_num = plot_lattice(adiag_gaps, fig_num,
                         write_file=os.path.join(result_dir, 't01_adiag_gaps.png'))
  fig_num = plot_lattice(adiag_gaps > 1, fig_num,
                         write_file=os.path.join(result_dir, 't01_adiag_gaps_thresh.png'))



  plt.show()

if __name__ == "__main__":
  main()
